-- This script file is designed to reproduce the bug in MariaDB.
-- It has been modified to be self-contained, so
-- everything in this file should access the single database...
USE `bird_ebird_test`;

DROP TABLE if exists counties_subset;
CREATE TABLE  counties_subset like counties;
INSERT  into counties_subset 
	select * from counties where
    IDCountyCode like "%GYM" or 
    IDCountyCode like "%LOV" or 
    IDCountyCode like "%RED" or 
    IDCountyCode like "%SOM" ;
     
DROP TABLE if exists sampling_subset;
CREATE  table sampling_subset like samplingEvents;
INSERT into sampling_subset
 select * from samplingEvents
	where IDLocation in 
	(select IDLocation from locations where IDCountyCode in
				(select IDCountyCode from counties_subset) )
	and   obsDate BETWEEN  '2021-01-01' and  '2021-03-31'  ;

ALTER TABLE  sampling_subset 
	-- ENGINE = InnoDB,   -- this avoids the bug
	ADD COLUMN `nSpecies` INT NOT NULL DEFAULT 0,
	ADD COLUMN `totalBirds` INT NOT NULL DEFAULT 0,
	ADD COLUMN `mon` SMALLINT NULL,
	ADD COLUMN `weekOfYear` SMALLINT NULL ,
	ADD INDEX `idx_month` (`mon` ASC) ,
	ADD INDEX `idx_woy` (`weekOfYear` ASC) ;

UPDATE sampling_subset
	set `mon` = month(obsDate),
		weekOfYear = week( obsDate, 5);

        
-- calculate some totals over the subset of data.
DROP temporary table if exists birdcounts;
CREATE  temporary table birdcounts
    select  IDSamplingEvent,  count(IDObservation) as nSpecies, max( birdCount) as max_of1_Species, sum(birdCount) as totalBirds, obsDate
		from Observations as o
		join sampling_subset as s  using (IDSamplingEvent) 
		GROUP BY ( IDSamplingEvent)  ;

update  sampling_subset as s 
		join  birdcounts as bc  using (IDSamplingEvent)
		set s.nSpecies = bc.nSpecies,
            s.totalBirds = bc.totalBirds;
-- and the monthly data split by Administratve region
DROP TABLE if exists month_summary_by_county;
CREATE TABLE month_summary_by_county
 SELECT `mon`,  IDCountyCode,   countyName AS county,  COUNT(IDSamplingEvent) AS nSurveys, SUM(nSpecies) AS nObs,  SUM(totalBirds) AS totBirds
    FROM  sampling_subset
        JOIN  `locations`  USING (IDLocation)
        JOIN  `counties_subset` USING (IDCountyCode)
GROUP BY `mon` , IDCountyCode
ORDER BY IDCountyCode , `mon`;

ALTER TABLE   `month_summary_by_county` 
    CHANGE COLUMN `mon` `mon` SMALLINT NOT NULL ,
    CHANGE COLUMN `IDCountyCode` `IDCountyCode` CHAR(12) NOT NULL COMMENT 'index into county (local govt) table.' ,
	CHANGE COLUMN `nSurveys` `nSurveys` INT NOT NULL ,
	CHANGE COLUMN `nObs` `nObs` INT NULL DEFAULT NULL ,
	CHANGE COLUMN `totBirds` `totBirds` INT NULL DEFAULT NULL ,
	ADD PRIMARY KEY (`mon`, `IDCountyCode`);

-- this code demonstrates what seems to be a bug in MariaDB.
-- The code within blocks A and B should create the same results, and sometimes do!
-- but sometimes not.
-- Block A seems always correct (ca 0.25sec), 
--    but block B sometimes gives a very truncated list (after 42 sec).
-- the final table should be the number of months times the number of counties selected.


-- code block A
drop temporary table if exists junk2;
create temporary table junk2
select mon, IDCountyCode,  sum(birdCount) as birds, count(IDSamplingEvent) as timesReported
	from `Observations` as ob
		JOIN
	`sampling_subset` as se USING (IDSamplingEvent)
		JOIN
        `locations` as loc using (IDLocation)
    where IDutax = 460		-- pick one species
    group by `mon`, IDCountyCode;

   select  mon,  IDCountyCode, birds, timesReported, timesReported/nSurveys * 100 as reportPct, birds/totBirds * 1000 as frac_per_thou 
   from junk2
    join
   month_summary_by_county as m USING (IDCountyCode, mon)
   order by  IDCountyCode, `mon`  ;

 -- code block B  
SELECT  mon,  IDCountyCode, birds, timesReported, timesReported/nSurveys * 100 as reportPct, birds/totBirds * 1000 as frac_per_thou from
  (
  select  mon, IDCountyCode,  sum(birdCount) as birds, count(IDSamplingEvent) as timesReported
	from `Observations` as ob
		JOIN
    `sampling_subset` as se USING (IDSamplingEvent)
		JOIN
        `locations` as loc using (IDLocation)
    where IDutax = 460
    group by `mon`, IDCountyCode
  ) as spec
   inner JOIN
   month_summary_by_county as m USING (IDCountyCode, mon)
   order by  IDCountyCode, `mon`  ;
   
-- now check the table and rerun the block B script.
-- check  table sampling_subset;
